%%
%% This is file `nnw.cls',
%%
%% Copyright (c) 2013,2014,2015,2016 CVUT FTS
%%
%% Authors: Eva Martincikova
%%          Jan Prikryl
%%
%% Changes:
%%   2015/02/21 v2.0 beta 2:
%%     * ignore \par command and hence all empty lines between
%%       \begin{abstract} and \end{abstract}
%%     * use `cite` package to provide visualy more appealing numeric
%%       citations
%%     * automatically capitalize the title
%%     * corrected a bug that caused incorrect baselineskip in title when
%%       using manual line breaks (\\), inspired by
%%       http://tex.stackexchange.com/questions/77909/space-between-lines
%%     * execute `twoside` option by default
%%   2015/10/20 v 2.0 beta 3:
%%     * added `a4paper` global option when loading `article` class to
%%       prevent problems with packages that assume `letter` paper by
%%       default (hyperref)
%%     * page dimensions are now specified using the `geometry` package
%%       for the same reason
%%   2016/04/28 v 2.0 beta 4:
%%     * use `hyperref` by default
%%     * new option "galley" produces print galleys in black&white
%%     * new option "tutorial" indicates a tutorial and provides an extra
%%       command for typesetting tutorial CV's
%%     * new option "nocite" prevents the `cite` package to be loaded in
%%       cases when one wants to use biblatex
%%   2016/05/03 v 2.0 beta 5:
%%     * patched incorrect handling of headings when used with lscape package
%%       by replacing TeX \centerline{} with \hfill{....}\hfill
%%   2016/xx/xx v 2.0 beta 6:
%%     * todo use biblatex with biber or bibtex, patch cite
%%     * todo new macros for typesetting the list of authors
%%     * todo new macros for typesetting tutorial CV's
%% --------------------------------------------------------------------------
\NeedsTeXFormat{LaTeX2e}[1995/12/01]
\ProvidesClass{nnw}
              [2016/05/03 v2.0 beta 5 Neural Network World article class]
%% --------------------------------------------------------------------------
%% Custom package options
\newif\ifcit@
\cit@true
\DeclareOption{nocite}{
  \cit@false
}

\newif\ifgall@y
\gall@yfalse
\DeclareOption{galley}{
  \gall@ytrue
}
\newif\iftut@
\tut@false
\DeclareOption{tutorial}{
  \tut@true
}
%% Fallback
\DeclareOption*{
  \ClassWarning{myclass}{Unknown option '\CurrentOption'}
}
%% --------------------------------------------------------------------------
%% Process given options
\ProcessOptions\relax
%% --------------------------------------------------------------------------
%% Load defaults from 'article.cls'
\LoadClass[twoside,a4paper]{article}
%% --------------------------------------------------------------------------
%% Pass different options to hyperref for galley (all links black) and
%% electronic version (coloured links without frames)
\ifgall@y
  \PassOptionsToPackage{hidelinks}{hyperref}
\else
  \PassOptionsToPackage{colorlinks=true}{hyperref}
\fi
%% Preload packages that are important to our authors
\RequirePackage{amsmath}
\RequirePackage{amsthm}
\RequirePackage{amssymb}
\RequirePackage{amsfonts}
\RequirePackage{mathtools} % This is a patch to some errors in AMS
\RequirePackage{booktabs}
%% Typesetting algorithms
\RequirePackage{algorithmic} % algorithm typesetting
\RequirePackage{algorithm} % float environment `algorithm`
%% Typesetting of SI units and decimal-point alignment of numbers
\RequirePackage{siunitx}
%% Use wrapfig for CVs in tutorials, but load it only for tutorials
\iftut@%
\RequirePackage{wrapfig}
\fi
%% More visually appealing citations cannot be used with biblatex
%% which has its own formatting routine that achieves more or less the same.
\ifcit@%
\RequirePackage{cite}
\fi
%% --------------------------------------------------------------------------
%%                                                            Page dimensions
%% --------------------------------------------------------------------------
%% The original specifiactions from tereza.cls is:
%%   \textwidth 126mm
%%   \textheight 200mm
%%   \oddsidemargin 18mm
%%   \evensidemargin 18mm
%% This seem to produce text that is not horizontally centered on the page,
%% we will therefore use 42-126-42 mm setup (this sums up to 210mm, the
%% width of an A4 page).
\RequirePackage[a4paper,textwidth=126mm,textheight=200mm,left=42mm,right=42mm]{geometry}
%% --------------------------------------------------------------------------
%%                                                                   Hyperref
%% --------------------------------------------------------------------------
%% Clickable DOI
\newcommand{\doi}[1]{doi:~\href{http://dx.doi.org/#1}{\nolinkurl{#1}}}
%% Conditional support for PDF and DVI mode with `hyperref`
\RequirePackage{ifpdf}
\ifpdf
  %% The `graphicx` package behaves differently in PDF and DVI mode
  \RequirePackage[pdftex]{graphicx}
  %% Needed for colouring links in PDF mode
  \usepackage{xcolor}
  \definecolor{cvutbase}{RGB}{0,99,168}
  \definecolor{cvutgreen}{RGB}{0,168,99}
  %% This will typeset the electronic version or galley
  \usepackage[pdftex,breaklinks=true,linkcolor=cvutbase,urlcolor=blue,citecolor=cvutgreen,plainpages=false,setpagesize=false]{hyperref}
\else
  %% The `graphicx` package behaves differently in PDF and DVI mode
  \RequirePackage[dvips]{graphicx}
  %% B&W in DVI mode
  %% The unfortunate side effect of using DVI output is the lack of proper support
  %% for breaking long links (url, doi)
  \usepackage[dvips,breaklinks=true,hidelinks,setpagesize=false]{hyperref}
  %% This is needed in DVI->PS mode to break links
  %% \usepackage[anythingbreaks]{breakurl}
  \usepackage{breakurl}
  %% \nolinkurl is ignored by breakurl package, use a hack
  \def\nolinkurl#1{\burl{#1}}
\fi
%% --------------------------------------------------------------------------
%%                                                                   Headings
%% --------------------------------------------------------------------------
%%
%% This is defined in 'article.cls' but we need different
%% headings style.
%% TODO: Use standard packages for headings?
\def\ps@myheadings{%
%% Original:
%%    \let\@oddfoot\@empty\let\@evenfoot\@empty
%%    \def\@evenhead{\thepage\hfil\slshape\leftmark}%
%%    \def\@oddhead{{\slshape\rightmark}\hfil\thepage}%
%%    \let\@mkboth\@gobbletwo
%%    \let\sectionmark\@gobble
%%    \let\subsectionmark\@gobble
  \def\@oddhead{\hfill{\footnotesize\bfseries\rightmark}\hfill}
  \def\@oddfoot{\hfill{\bfseries\thepage}}
  \def\@evenhead{\hfill{\footnotesize\bfseries\leftmark}\hfill}
  \def\@evenfoot{{\bfseries\thepage}\hfill}
  \def\sectionmark##1{}
  \def\subsectionmark##1{}
}
\def\ps@plain{%
  \let\@mkboth\@gobbletwo
  \def\@oddfoot{\hfill{\bfseries\thepage}}
  \def\@evenfoot{{\bfseries\thepage}\hfill}
  \def\@oddhead{}
  \def\@evenhead{}%
}
\def\ps@firstheadings{%
  \def\@oddhead{\hfill\leavevmode\includegraphics[width=2cm]{nnw_logo}\hfill}
  \def\@evenhead{}
  \def\@oddfoot{{\bfseries\copyright CTU FTS 2023 \hfill\thepage}}
  \def\@evenfoot{}%
}

%% --------------------------------------------------------------------------
%%                                                                 Title page
%% --------------------------------------------------------------------------
%%
\renewcommand\maketitle{%
  \par
  \begingroup
    \def\thefootnote{\fnsymbol{footnote}}
    \def\@makefnmark{\rlap{\@textsuperscript{\normalfont\@thefnmark}}}%
    \long\def\@makefntext##1{%
      \parindent 1em
      \noindent
      \hb@xt@1.8em{%
        \hss\@textsuperscript{\normalfont\@thefnmark}}##1}%
    \if@twocolumn
      \twocolumn[\@maketitle]
    \else
      \newpage
      \global\@topnum\z@
      \@maketitle
    \fi
    \thispagestyle{firstheadings}
    \@thanks
 \endgroup
 %
 \setcounter{footnote}{0}
 \let\maketitle\relax
 \let\@maketitle\relax
 \gdef\@thanks{}
 \gdef\@author{}
 \gdef\@title{}
 \let\thanks\relax%
}
\def\@maketitle{%
  \newpage
  \null
  \hrule height 0.5 pt depth 0.5 pt
  \smallskip
  \begin{center}
    \Large \bfseries \uppercase\expandafter{\@title}
  \end{center}
  {%
    \parfillskip 0pt
    \parindent 0pt
    \begin{center}
      {\itshape \@author}
    \end{center}
  }%

  \smallskip
  \hrule height 0.5 pt depth 0.5 pt
  \vspace{0.8cm}%
}

%% NNW uses MS-Word-like formatting of section numbers: Every section number
%% ends with a dot, every lower levels are without the dot. This is one
%% possible implementation, insipred by http://tex.stackexchange.com/questions/29631/
%% which was in turn inspired by UK TUG FAQ:
%% We will patch the \@seccntformat and introduce our own counter formatting
%% commands for every level
\renewcommand{\@seccntformat}[1]{\csname nnw@#1\endcsname\quad}
\newcommand{\nnw@section}{{\thesection.}}
\newcommand{\nnw@subsection}{{\thesubsection}}
\newcommand{\nnw@subsubsection}{{\thesubsubsection}}

\def\titlepage{\@restonecolfalse\if@twocolumn\@restonecoltrue\onecolumn
 \else \newpage \fi \thispagestyle{firstheadings}\c@page\z@}
\def\endtitlepage{\if@restonecol\twocolumn \else \newpage \fi}

%% NNW style requires tables to be numbered in Roman numerals
\renewcommand\thetable{\@Roman\c@table}

%% Caption code is again copy-and-pasted from 'article.cls'. NNW captions
%% use boldface font for item designator and counter, and the caption itself
%% is typeset in italics. This requires replacing original references to
%% parameters #1 and #2 with \textbf{#1} and \textit{#2}.
%% TODO: Check the 'caption' package.
\long\def\@makecaption#1#2{%
  \vskip\abovecaptionskip
  \sbox\@tempboxa{\textbf{#1} \textit{#2}}%
  \ifdim \wd\@tempboxa >\hsize
    \textbf{#1} \textit{#2}\par
  \else
    \global \@minipagefalse
    \hb@xt@\hsize{\hfil\box\@tempboxa\hfil}%
  \fi
  \vskip\belowcaptionskip}

%% Make the part counter heading wider (from 2 to 2.25 em)
%% Again, there are packages that do this automatically (tocloft)
%% and it is questionable where we actually use the \part command.
%% TODO: Check if this code can be omitted, replaced with \patchcmd
%% from etoolbox or with tocloft package.
\renewcommand*\l@part[2]{%
  \ifnum \c@tocdepth >-2\relax
    \addpenalty\@secpenalty
    \addvspace{2.25em \@plus\p@}%
    \begingroup
      \parindent \z@ \rightskip \@pnumwidth
      \parfillskip -\@pnumwidth
      {\leavevmode
       \large \bfseries #1\hfil \hb@xt@\@pnumwidth{\hss #2}}\par
       \nobreak
       \if@compatibility
         \global\@nobreaktrue
         \everypar{\global\@nobreakfalse\everypar{}}%
      \fi
    \endgroup
  \fi}

%% Bibliography environment with smaller font.
%% This is a copy-and-paste from 'article.cls' with \footnotesize
%% and \normalsize inserted in.
\renewenvironment{thebibliography}[1]
     {\section*{\refname}%
        \@mkboth{\MakeUppercase\refname}{\MakeUppercase\refname}%
      \footnotesize%
      \list{\@biblabel{\@arabic\c@enumiv}}%
           {\settowidth\labelwidth{\@biblabel{#1}}%
            \leftmargin\labelwidth
            \advance\leftmargin\labelsep
            \@openbib@code
            \usecounter{enumiv}%
            \let\p@enumiv\@empty
            \renewcommand\theenumiv{\@arabic\c@enumiv}}%
      \sloppy
      \clubpenalty4000
      \@clubpenalty \clubpenalty
      \widowpenalty4000%
      \sfcode`\.\@m}
     {\def\@noitemerr
       {\@latex@warning{Empty `thebibliography' environment}}%
      \endlist\normalsize}

%% Abstract
%% We ignore paragraph breaks inside the abstract environment. This is
%% not considered to be a very good practice though.
\renewenvironment{abstract}%
  {\noindent{\bfseries\abstractname:}\begingroup\let\par\relax}
  {\endgroup\medskip}

%% This contains both text and formatting. The formatting part should be
%% part of \maketitle command.
%% TODO: Change it.
\newcommand{\key}[1]{%
  \par%
  \medskip%
  \noindent{%
  \newsavebox{\kwbox}%
  \sbox{\kwbox}{Key words: }%
  \newlength{\kwlen}%
  \settowidth{\kwlen}{\usebox{\kwbox}}%
  \hangindent \kwlen
  \hangafter 1
  \usebox{\kwbox}\textit{#1}\par}}
\newcommand{\rec}[1]{\medskip\par\noindent \textit{Received: #1}}
\newcommand{\rev}[1]{\par\noindent \textit{Revised and accepted: #1}\bigskip}

\newcommand{\obr}[2]{\begin{center}\leavevmode\epsfxsize=#2cm\epsffile{#1}\end{center}}
\newcommand{\zvets}{\refstepcounter{footnote}}

%% Acknowledgement
\newcommand\ackn{\subsection*{Acknowledgement}}

%% Define our own \figurename and \tablename
%% A simple redefinition does not suffice as in the case of using babel or
%% polygossia the names are being redefined by the package. We will therefore
%% first create a private command that redefines both names ...
\newcommand\nnwflo@tabrv{%
  \renewcommand{\figurename}{Fig.}%
  \renewcommand{\tablename}{Tab.}%
}
%% ... and then use the command in a form that is applicable to environment
%% where babel is used and to environment without babel.
\newcommand\nnwflo@thook{%
  \@ifpackageloaded{babel}{%
    \addto\captionsenglish{\nnwflo@tabrv}%
  }{%
    \nnwflo@tabrv%
  }%
}
%% Finally, we have to defer the babel-aware patching of \figurename and
%% \tablename to the beginning of document as at this moment we do not know
%% if the author is going to use babel or not.
%% TODO: As soon as some package redefines the \figurename and \tablename
%% using the \AtBeginDocument hook, the package will take a precedence.
\AtBeginDocument{%
  \nnwflo@thook
}

%% Theorem and lemma definitions
\theoremstyle{plain}
\newtheorem{thm}{Theorem}
\newtheorem{lem}[thm]{Lemma}
\newtheorem{prop}[thm]{Proposition}
\newtheorem*{cor}{Corollary}

\theoremstyle{definition}
\newtheorem{defn}{Definition}
\newtheorem{conj}{Conjecture}
\newtheorem{example}{Example}

\theoremstyle{remark}
\newtheorem*{rem}{Remark}
\newtheorem*{note}{Note}
\newtheorem{case}{Case}

\endinput
%%
%% End of file 'nnw.cls'.
